@@ -1,12 +1,12 @@
 #      Disassembler.pm
 #
 #      Copyright (c) 1996 Malcolm Beattie
-#      Copyright (c) 2008,2009,2010,2011 Reini Urban
+#      Copyright (c) 2008,2009,2010,2011,2012 Reini Urban
 #
 #      You may distribute under the terms of either the GNU General Public
 #      License or the Artistic License, as specified in the README file.
 
-$B::Disassembler::VERSION = '1.10';
+$B::Disassembler::VERSION = '1.12';
 
 package B::Disassembler::BytecodeStream;
 
@@ -222,6 +222,7 @@ use strict;
 use B::Asmdata qw(%insn_data @insn_name);
 use Opcode qw(opset_to_ops full_opset);
 use Config qw(%Config);
+use B::Concise;
 
 BEGIN {
   if ( $] < 5.009 ) {
@@ -233,6 +234,7 @@ BEGIN {
 }
 
 my $ix;
+my $opname;
 our @opname = opset_to_ops(full_opset);
 our (
   $magic,   $archname, $blversion, $ivsize,
@@ -240,10 +242,12 @@ our (
 );
 						# >=5.12
 our  @svnames = ("NULL");			# 0
-push @svnames, "BIND"   if $] >= 5.009;	# 1
+push @svnames, "BIND"   if $] >= 5.009 and $] < 5.019002; # 1
 push @svnames, ("IV", "NV");			# 2,3
 push @svnames, "RV"     if $] < 5.011;		#
-push @svnames, ("PV", "PVIV", "PVNV", "PVMG");	# 4-7
+push @svnames, "PV";
+push @svnames, "INVLIST" if $] >= 5.019002;     # 4
+push @svnames, ("PVIV", "PVNV", "PVMG");	# 4-7
 push @svnames, "BM"     if $] < 5.009;
 push @svnames, "REGEXP" if $] >= 5.011;	# 8
 push @svnames, "GV"     if $] >= 5.009;	# 9
@@ -315,8 +319,9 @@ sub print_insn {
     if ( $insn eq 'newopx' or $insn eq 'ldop' and $] > 5.007) {
       my $type = $arg >> 7;
       my $size = $arg - ( $type << 7 );
-      $arg .= sprintf( " \t# size:%d, type:%d %s", $size, $type ) if $comment;
-      printf "\n# [%s %d]\n", $opname[$type], $ix++ if $comment;
+      $arg .= sprintf( " \t# size:%d, type:%d %s", $size, $type) if $comment;
+      $opname = $opname[$type];
+      printf "\n# [%s %d]\n", $opname, $ix++;
     }
     elsif ( !$comment ) {
       ;
@@ -331,6 +336,10 @@ sub print_insn {
       $arg .= $comment if $comment ne '1';
       printf "\n# [%s %d]\n", $svnames[$type], $ix++;
     }
+    elsif ( $insn eq 'newpadlx' ) {
+      $arg .= "\t# " . $comment if $comment ne '1';
+      printf "\n# [%s %d]\n", "PADLIST", $ix++;
+    }
     elsif ( $insn eq 'gv_stashpvx' ) {
       $arg .= "\t# " . $comment if $comment ne '1';
       printf "\n# [%s %d]\n", "STASH", $ix++;
@@ -352,7 +361,14 @@ sub print_insn {
       $arg .= sprintf( "\t# '%s'", chr($arg) );
     }
     elsif ( $insn =~ /_flags/ ) {
-      $arg .= sprintf( "\t# 0x%x", $arg );
+      my $f = $arg;
+      $arg .= sprintf( "\t# 0x%x", $f ) if $comment;
+      $arg .= " ".B::Concise::op_flags($f) if $insn eq 'op_flags' and $comment;
+    }
+    elsif ( $comment and $insn eq 'op_private' ) {
+      my $f = $arg;
+      $arg .= sprintf( "\t# 0x%x", $f );
+      $arg .= " ".B::Concise::private_flags($opname, $f);
     }
     elsif ( $insn eq 'op_type' and $] < 5.007 ) {
       my $type = $arg;
@@ -407,6 +423,7 @@ sub disassemble_fh {
       warn "Illegal instruction code $c at stream offset $pos.\n";
     }
     $getmeth = $insn_data{$insn}->[2];
+    #warn "EOF at $insn $getmeth" if $fh->eof();
     $arg     = $fh->$getmeth();
     if ( defined($arg) ) {
       &$out( $insn, $arg, $verbose );
